
#ifndef ROOT_TShape
#define ROOT_TShape

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TShape                                                               //
//                                                                      //
// Basic shape class                                                    //
//                                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TNamed
//*KEEP,TNamed.
#include "TNamed.h"
//*KEND.
#endif

#ifndef ROOT_TMaterial
//*KEEP,TMaterial.
#include "TMaterial.h"
//*KEND.
#endif

#ifndef ROOT_TAttLine
//*KEEP,TAttLine.
#include "TAttLine.h"
//*KEND.
#endif

#ifndef ROOT_TAttFill
//*KEEP,TAttFill.
#include "TAttFill.h"
//*KEND.
#endif

#ifndef ROOT_TAtt3D
//*KEEP,TAtt3D.
#include "TAtt3D.h"
//*KEND.
#endif

#ifndef ROOT_X3DBuffer
//*KEEP,X3DBuffer,T=C.
#include "X3DBuffer.h"
//*KEND.
#endif

#ifndef ROOT_TPolyLine3D
//*KEEP,TPolyLine3D.
#include "TPolyLine3D.h"
//*KEND.
#endif

class TNode;

class TShape : public TNamed, public TAttLine, public TAttFill, public TAtt3D {

protected:
   Int_t           fNumber;      //Shape number
   Int_t           fVisibility;  //Visibility flag
   TMaterial       *fMaterial;   //Pointer to material
   Int_t           ShapeDistancetoPrimitive(Int_t numPoints, Int_t px, Int_t py);

public:
                   TShape();
                   TShape(const char *name, const char *title, const char *material);
   virtual         ~TShape();
   TMaterial       *GetMaterial() {return fMaterial;}
   virtual Int_t   GetNumber() {return fNumber;}
           Int_t   GetVisibility() {return fVisibility;}
   virtual void    Paint(Option_t *option="");
   virtual void    PaintGLPoints(Float_t *vertex);
   virtual void    PaintShape(X3DBuffer *buff, Bool_t rangeView=kFALSE);
   virtual void    SetName(const char *name);
   virtual void    SetPoints(Float_t *buffer);
   virtual void    SetVisibility(Int_t vis) {fVisibility = vis;} // *MENU*

   ClassDef(TShape,1)  //Basic shape
};

R__EXTERN TNode *gNode;

inline void TShape::PaintGLPoints(Float_t *) { }
inline void TShape::SetName(const char *) { }

#endif

